# Compiled code
useDynLib(sumer, .registration = TRUE)

# Exports
export(as.cuneiform)
export(as.sign_name)
export(convert_to_dictionary)
export(info)
export(look_up)
export(make_dictionary)
export(read_dictionary)
export(read_translated_text)
export(save_dictionary)
export(split_sumerian)
export(ngram_frequencies)
export(sign_grammar)
export(plot_sign_grammar)
export(prior_probs)
export(grammar_probs)
export(skeleton)
export(mark_ngrams)
export(guess_substr_info)
export(translate)
export(apply_translation_rules)
export(grammatical_structure)
export(fill_substr_info)
export(merge_dictionaries)
export(translation_context)
export(translate_line)

# S3 methods
S3method(as.cuneiform, character)
S3method(as.cuneiform, default)
S3method(as.sign_name, character)
S3method(as.sign_name, default)
S3method(print, info)
S3method(print, cuneiform)
S3method(print, sign_name)
S3method(print, look_up)
S3method(print, skeleton)
S3method(print, grammatical_structure)

# Imports from stringr
importFrom(stringr, str_c)
importFrom(stringr, str_count)
importFrom(stringr, str_detect)
importFrom(stringr, str_ends)
importFrom(stringr, str_escape)
importFrom(stringr, str_remove)
importFrom(stringr, str_remove_all)
importFrom(stringr, str_replace)
importFrom(stringr, str_replace_all)
importFrom(stringr, str_locate_all)
importFrom(stringr, str_length)
importFrom(stringr, str_split)
importFrom(stringr, str_split_1)
importFrom(stringr, str_split_fixed)
importFrom(stringr, str_squish)
importFrom(stringr, str_starts)
importFrom(stringr, str_sub)
importFrom(stringr, str_trim)
importFrom(stringr, regex)
importFrom(stringr, fixed)

# Imports from utils
importFrom(utils, read.csv2)
importFrom(utils, read.table)
importFrom(utils, write.table)

# Imports from stats
importFrom(stats, aggregate)
importFrom(stats, setNames)

# Import .data pronoun for ggplot2 aes()
importFrom(rlang, .data)

# Imports from officer
importFrom(officer, read_docx)
importFrom(officer, docx_summary)

# Imports from xml2
importFrom(xml2, xml_find_all)
importFrom(xml2, xml_remove)

# Imports from cli
importFrom(cli, col_blue)
importFrom(cli, col_green)
importFrom(cli, col_red)
importFrom(cli, make_ansi_style)

# Imports from grDevices / tools (used in plot_sign_grammar)
importFrom(grDevices, dev.off)
importFrom(tools, file_ext)
